/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('account', function () {

  juju.components.Account = React.createClass({

    propTypes: {
      acl: React.PropTypes.object.isRequired,
      user: React.PropTypes.object.isRequired,
      users: React.PropTypes.object.isRequired
    },

    getInitialState: function () {
      return {
        credentials: [],
        loadingCredentials: true
      };
    },

    componentWillMount: function () {
      this._getCredentials();
    },

    /**
      Get the credentials from the API.
       @method _getCredentials
    */
    _getCredentials: function () {
      console.log('Fetching credentials is not implemented');
    },

    /**
      Generate the rows of credentials.
       @method _generateCredentials
      @returns {Array} The list of credential components.
    */
    _generateCredentials: function () {
      var credentials = this.state.credentials;
      if (credentials.length === 0) {
        return;
      }
      var components = [];
      var classes = {
        'user-profile__entity': true,
        'user-profile__list-row': true
      };
      credentials.forEach(credential => {
        var path = credential.path;
        var parts = path.split('/');
        var owner = parts[0];
        var name = parts[1];
        components.push(React.createElement(
          juju.components.ExpandingRow,
          {
            classes: classes,
            key: path },
          React.createElement(
            'div',
            null,
            React.createElement(
              'span',
              { className: 'user-profile__list-col three-col' },
              name
            ),
            React.createElement(
              'span',
              { className: 'user-profile__list-col three-col' },
              '--'
            ),
            React.createElement(
              'span',
              { className: 'user-profile__list-col three-col' },
              owner
            ),
            React.createElement(
              'span',
              { className: 'user-profile__list-col three-col last-col' },
              '--'
            )
          ),
          React.createElement(
            'div',
            null,
            React.createElement(
              'div',
              { className: 'expanding-row__expanded-header twelve-col' },
              React.createElement(
                'div',
                { className: 'nine-col no-margin-bottom' },
                name
              ),
              React.createElement(
                'div',
                { className: 'expanding-row__expanded-header-action ' + 'three-col last-col no-margin-bottom' },
                React.createElement(juju.components.GenericButton, {
                  action: this._handleDestroyCredential.bind(this, name),
                  disabled: this.props.acl.isReadOnly(),
                  type: 'inline-base',
                  title: 'Destroy' }),
                React.createElement(juju.components.GenericButton, {
                  action: this._handleEditCredential,
                  disabled: this.props.acl.isReadOnly(),
                  type: 'inline-neutral',
                  title: 'Edit' })
              )
            ),
            React.createElement(
              'div',
              { className: 'expanding-row__expanded-content twelve-col ' + 'no-margin-bottom' },
              React.createElement(
                'ul',
                { className: 'user-profile__list twelve-col' },
                React.createElement(
                  'li',
                  { className: 'user-profile__list-header twelve-col' },
                  React.createElement(
                    'span',
                    { className: 'user-profile__list-col three-col' },
                    'Model'
                  ),
                  React.createElement(
                    'span',
                    { className: 'user-profile__list-col three-col' },
                    'Units'
                  ),
                  React.createElement(
                    'span',
                    { className: 'user-profile__list-col three-col' },
                    'Owner'
                  ),
                  React.createElement(
                    'span',
                    { className: 'user-profile__list-col three-col last-col' },
                    'Region'
                  )
                ),
                React.createElement(
                  'li',
                  { className: 'user-profile__list-row twelve-col' },
                  React.createElement(
                    'span',
                    { className: 'user-profile__list-col three-col' },
                    'Callisto'
                  ),
                  React.createElement(
                    'span',
                    { className: 'user-profile__list-col three-col' },
                    '4'
                  ),
                  React.createElement(
                    'span',
                    { className: 'user-profile__list-col three-col' },
                    'Spinach'
                  ),
                  React.createElement(
                    'span',
                    { className: 'user-profile__list-col three-col last-col' },
                    'US (East)'
                  )
                ),
                React.createElement(
                  'li',
                  { className: 'user-profile__list-row twelve-col' },
                  React.createElement(
                    'span',
                    { className: 'user-profile__list-col three-col' },
                    'Callisto'
                  ),
                  React.createElement(
                    'span',
                    { className: 'user-profile__list-col three-col' },
                    '4'
                  ),
                  React.createElement(
                    'span',
                    { className: 'user-profile__list-col three-col' },
                    'Spinach'
                  ),
                  React.createElement(
                    'span',
                    { className: 'user-profile__list-col three-col last-col' },
                    'US (East)'
                  )
                )
              )
            )
          )
        ));
      });
      return components;
    },

    /**
      Generate the spinner if the credentials are loading.
       @method _generateSpinner
      @returns {Array} The spinner component.
    */
    _generateSpinner: function () {
      if (this.state.loadingCredentials) {
        return React.createElement(juju.components.Spinner, null);
      }
    },

    /**
      Handle destroying a credential.
       @method _handleDestroyCredential
      @param {String} name The name of the credential.
    */
    _handleDestroyCredential: function (name) {
      console.log('Removal of credentials is not implemented');
    },

    /**
      Handle editing a credential.
       @method _handleEditCredential
    */
    _handleEditCredential: function () {
      console.log('Editing of credentials is not implemented');
    },

    /**
      Handle Signing out.
       @method _handleSignOut
    */
    _handleSignOut: function () {
      console.log('Signing out is not implemented');
    },

    render: function () {
      var username = this.props.user && this.props.user.usernameDisplay;
      var links = [{
        label: '(Primary account)',
        type: 'light'
      }, {
        action: this._handleSignOut,
        label: 'Sign out'
      }];
      return React.createElement(
        juju.components.Panel,
        {
          instanceName: 'account',
          visible: true },
        React.createElement(
          'div',
          { className: 'twelve-col' },
          React.createElement(
            'div',
            { className: 'inner-wrapper' },
            React.createElement(juju.components.UserProfileHeader, {
              users: this.props.users,
              avatar: '',
              links: links,
              username: username }),
            React.createElement(
              'h2',
              { className: 'account__title twelve-col' },
              'Account management'
            ),
            React.createElement(
              'h3',
              { className: 'account__title2 twelve-col' },
              'Cloud credentials',
              React.createElement(
                'ul',
                { className: 'account__title-links' },
                React.createElement(
                  'li',
                  { className: 'account__title-link' },
                  'Add'
                ),
                React.createElement(
                  'li',
                  { className: 'account__title-bullet' },
                  '\xA0\u2022\xA0'
                ),
                React.createElement(
                  'li',
                  { className: 'account__title-link' },
                  'Edit defaults'
                )
              )
            ),
            React.createElement(
              'form',
              { className: 'twelve-col' },
              React.createElement(
                'div',
                { className: 'six-col' },
                React.createElement(
                  'label',
                  { className: 'deployment-panel__label',
                    htmlFor: 'default-credential' },
                  'Default credential'
                ),
                React.createElement('input', { className: 'deployment-panel__input',
                  id: 'default-credential',
                  type: 'text' })
              ),
              React.createElement(
                'div',
                { className: 'six-col last-col' },
                React.createElement(
                  'label',
                  { className: 'deployment-panel__label',
                    htmlFor: 'default-region' },
                  'Default region'
                ),
                React.createElement('input', { className: 'deployment-panel__input',
                  id: 'default-region',
                  type: 'text' })
              )
            ),
            React.createElement(
              'ul',
              { className: 'user-profile__list twelve-col' },
              React.createElement(
                'li',
                { className: 'user-profile__list-header twelve-col' },
                React.createElement(
                  'span',
                  { className: 'user-profile__list-col three-col' },
                  'Credential name'
                ),
                React.createElement(
                  'span',
                  { className: 'user-profile__list-col three-col' },
                  'No. of models'
                ),
                React.createElement(
                  'span',
                  { className: 'user-profile__list-col three-col' },
                  'Owner'
                ),
                React.createElement(
                  'span',
                  { className: 'user-profile__list-col three-col last-col' },
                  'Provider'
                )
              ),
              this._generateSpinner(),
              this._generateCredentials()
            )
          )
        )
      );
    }

  });
}, '', {
  requires: ['expanding-row', 'loading-spinner', 'panel-component', 'svg-icon', 'user-profile-entity', 'user-profile-header']
});