/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

describe('Account', () => {
  var acl, users;

  beforeAll(done => {
    // By loading this file it adds the component to the juju components.
    YUI().use('account', () => {
      done();
    });
  });

  beforeEach(() => {
    acl = { isReadOnly: sinon.stub().returns(false) };
    users = {
      charmstore: {
        user: 'test-owner',
        usernameDisplay: 'test owner'
      }
    };
  });

  it('renders the loading state', () => {
    var component = jsTestUtils.shallowRender(React.createElement(juju.components.Account, {
      acl: acl,
      deleteTemplate: sinon.stub(),
      listTemplates: sinon.stub(),
      user: users.charmstore,
      users: users }), true);
    var instance = component.getMountedInstance();
    var output = component.getRenderOutput();
    var links = [{
      label: '(Primary account)',
      type: 'light'
    }, {
      action: instance._handleSignOut,
      label: 'Sign out'
    }];
    var expected = React.createElement(
      juju.components.Panel,
      {
        instanceName: 'account',
        visible: true },
      React.createElement(
        'div',
        { className: 'twelve-col' },
        React.createElement(
          'div',
          { className: 'inner-wrapper' },
          React.createElement(juju.components.UserProfileHeader, {
            users: users,
            avatar: '',
            links: links,
            username: 'test owner' }),
          React.createElement(
            'h2',
            { className: 'account__title twelve-col' },
            'Account management'
          ),
          React.createElement(
            'h3',
            { className: 'account__title2 twelve-col' },
            'Cloud credentials',
            React.createElement(
              'ul',
              { className: 'account__title-links' },
              React.createElement(
                'li',
                { className: 'account__title-link' },
                'Add'
              ),
              React.createElement(
                'li',
                { className: 'account__title-bullet' },
                '\xA0\u2022\xA0'
              ),
              React.createElement(
                'li',
                { className: 'account__title-link' },
                'Edit defaults'
              )
            )
          ),
          React.createElement(
            'form',
            { className: 'twelve-col' },
            React.createElement(
              'div',
              { className: 'six-col' },
              React.createElement(
                'label',
                { className: 'deployment-panel__label',
                  htmlFor: 'default-credential' },
                'Default credential'
              ),
              React.createElement('input', { className: 'deployment-panel__input',
                id: 'default-credential',
                type: 'text' })
            ),
            React.createElement(
              'div',
              { className: 'six-col last-col' },
              React.createElement(
                'label',
                { className: 'deployment-panel__label',
                  htmlFor: 'default-region' },
                'Default region'
              ),
              React.createElement('input', { className: 'deployment-panel__input',
                id: 'default-region',
                type: 'text' })
            )
          ),
          React.createElement(
            'ul',
            { className: 'user-profile__list twelve-col' },
            React.createElement(
              'li',
              { className: 'user-profile__list-header twelve-col' },
              React.createElement(
                'span',
                { className: 'user-profile__list-col three-col' },
                'Credential name'
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col three-col' },
                'No. of models'
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col three-col' },
                'Owner'
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col three-col last-col' },
                'Provider'
              )
            ),
            React.createElement(juju.components.Spinner, null),
            undefined
          )
        )
      )
    );
    assert.deepEqual(output, expected);
  });

  // It's not currently possible to fetch credentials so this is being
  // skipped until that functionality is re-enabled.
  xit('renders the credentials', () => {
    var listTemplates = sinon.stub().callsArgWith(0, null, [{ path: 'spinach/test-model' }]);
    var component = jsTestUtils.shallowRender(React.createElement(juju.components.Account, {
      acl: acl,
      deleteTemplate: sinon.stub(),
      listTemplates: listTemplates,
      user: users.charmstore,
      users: users }), true);
    var instance = component.getMountedInstance();
    instance.componentWillMount();
    var output = component.getRenderOutput();
    var credentials = output.props.children.props.children.props.children[4];
    var expected = React.createElement(
      'ul',
      { className: 'user-profile__list twelve-col' },
      React.createElement(
        'li',
        { className: 'user-profile__list-header twelve-col' },
        React.createElement(
          'span',
          { className: 'user-profile__list-col three-col' },
          'Credential name'
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col three-col' },
          'No. of models'
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col three-col' },
          'Owner'
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col three-col last-col' },
          'Provider'
        )
      ),
      undefined,
      [React.createElement(
        juju.components.ExpandingRow,
        {
          classes: {
            'user-profile__entity': true,
            'user-profile__list-row': true
          },
          key: 'spinach/test-model' },
        React.createElement(
          'div',
          null,
          React.createElement(
            'span',
            { className: 'user-profile__list-col three-col' },
            'test-model'
          ),
          React.createElement(
            'span',
            { className: 'user-profile__list-col three-col' },
            '--'
          ),
          React.createElement(
            'span',
            { className: 'user-profile__list-col three-col' },
            'spinach'
          ),
          React.createElement(
            'span',
            { className: 'user-profile__list-col three-col last-col' },
            '--'
          )
        ),
        React.createElement(
          'div',
          null,
          React.createElement(
            'div',
            { className: 'expanding-row__expanded-header twelve-col' },
            React.createElement(
              'div',
              { className: 'nine-col no-margin-bottom' },
              'test-model'
            ),
            React.createElement(
              'div',
              { className: 'expanding-row__expanded-header-action ' + 'three-col last-col no-margin-bottom' },
              React.createElement(juju.components.GenericButton, {
                action: credentials.props.children[2][0].props.children[1].props.children[0].props.children[1].props.children[0].props.action,
                disabled: false,
                type: 'inline-base',
                title: 'Destroy' }),
              React.createElement(juju.components.GenericButton, {
                action: instance._handleEditCredential,
                disabled: false,
                type: 'inline-neutral',
                title: 'Edit' })
            )
          ),
          React.createElement(
            'div',
            { className: 'expanding-row__expanded-content twelve-col ' + 'no-margin-bottom' },
            React.createElement(
              'ul',
              { className: 'user-profile__list twelve-col' },
              React.createElement(
                'li',
                { className: 'user-profile__list-header twelve-col' },
                React.createElement(
                  'span',
                  { className: 'user-profile__list-col three-col' },
                  'Model'
                ),
                React.createElement(
                  'span',
                  { className: 'user-profile__list-col three-col' },
                  'Units'
                ),
                React.createElement(
                  'span',
                  { className: 'user-profile__list-col three-col' },
                  'Owner'
                ),
                React.createElement(
                  'span',
                  { className: 'user-profile__list-col three-col last-col' },
                  'Region'
                )
              ),
              React.createElement(
                'li',
                { className: 'user-profile__list-row twelve-col' },
                React.createElement(
                  'span',
                  { className: 'user-profile__list-col three-col' },
                  'Callisto'
                ),
                React.createElement(
                  'span',
                  { className: 'user-profile__list-col three-col' },
                  '4'
                ),
                React.createElement(
                  'span',
                  { className: 'user-profile__list-col three-col' },
                  'Spinach'
                ),
                React.createElement(
                  'span',
                  { className: 'user-profile__list-col three-col last-col' },
                  'US (East)'
                )
              ),
              React.createElement(
                'li',
                { className: 'user-profile__list-row twelve-col' },
                React.createElement(
                  'span',
                  { className: 'user-profile__list-col three-col' },
                  'Callisto'
                ),
                React.createElement(
                  'span',
                  { className: 'user-profile__list-col three-col' },
                  '4'
                ),
                React.createElement(
                  'span',
                  { className: 'user-profile__list-col three-col' },
                  'Spinach'
                ),
                React.createElement(
                  'span',
                  { className: 'user-profile__list-col three-col last-col' },
                  'US (East)'
                )
              )
            )
          )
        )
      )]
    );
    assert.deepEqual(credentials, expected);
  });

  // It's not currently possible to fetch credentials so this is being
  // skipped until that functionality is re-enabled.
  xit('disables the buttons if in read only mode', () => {
    acl.isReadOnly = sinon.stub().returns(true);
    var component = jsTestUtils.shallowRender(React.createElement(juju.components.Account, {
      acl: acl,
      user: users.charmstore,
      users: users }), true);
    var instance = component.getMountedInstance();
    instance.componentWillMount();
    var output = component.getRenderOutput();
    var buttons = output.props.children.props.children.props.children[4].props.children[2][0].props.children[1].props.children[0].props.children[1];
    var expected = React.createElement(
      'div',
      { className: 'expanding-row__expanded-header-action ' + 'three-col last-col no-margin-bottom' },
      React.createElement(juju.components.GenericButton, {
        action: buttons.props.children[0].props.action,
        disabled: true,
        type: 'inline-base',
        title: 'Destroy' }),
      React.createElement(juju.components.GenericButton, {
        action: instance._handleEditCredential,
        disabled: true,
        type: 'inline-neutral',
        title: 'Edit' })
    );
    assert.deepEqual(buttons, expected);
  });
});