/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2016 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('budget-table', function () {

  juju.components.BudgetTable = React.createClass({
    propTypes: {
      acl: React.PropTypes.object.isRequired,
      allocationEditable: React.PropTypes.bool,
      extraInfo: React.PropTypes.object,
      listPlansForCharm: React.PropTypes.func,
      plansEditable: React.PropTypes.bool,
      services: React.PropTypes.array.isRequired,
      showExtra: React.PropTypes.bool,
      withPlans: React.PropTypes.bool
    },

    /**
     Generate the list of services.
      @method _generateServices
     @returns {Array} The list of services.
    */
    _generateServices: function () {
      return this.props.services.map((service, i) => {
        return React.createElement(juju.components.BudgetTableRow, {
          acl: this.props.acl,
          key: i,
          allocationEditable: this.props.allocationEditable,
          extraInfo: this.props.extraInfo && this.props.extraInfo[service.get('id')],
          listPlansForCharm: this.props.listPlansForCharm,
          plansEditable: this.props.plansEditable,
          service: service,
          showExtra: this.props.showExtra,
          withPlans: this.props.withPlans });
      });
    },

    /**
      Generate plan headers.
       @method _generatePlanHeaders
      @returns {Object} The plan headers markup.
    */
    _generatePlanHeaders: function () {
      if (!this.props.withPlans) {
        return;
      }
      const plansEditable = this.props.plansEditable;
      return React.createElement(
        'div',
        null,
        React.createElement(
          'div',
          { className: 'three-col' },
          'Details'
        ),
        React.createElement(
          'div',
          { className: plansEditable ? 'one-col' : 'two-col' },
          'Usage'
        ),
        React.createElement(
          'div',
          { className: plansEditable ? 'one-col' : 'two-col' },
          'Allocation'
        ),
        React.createElement(
          'div',
          { className: 'one-col last-col' },
          'Spend'
        )
      );
    },

    render: function () {
      return React.createElement(
        'div',
        { className: 'budget-table' },
        React.createElement(
          'div',
          { className: 'budget-table__row-header twelve-col' },
          React.createElement(
            'div',
            { className: 'three-col' },
            'Name'
          ),
          React.createElement(
            'div',
            { className: 'one-col' },
            'Units'
          ),
          this._generatePlanHeaders()
        ),
        this._generateServices()
      );
    }

  });
}, '0.1.0', {
  requires: ['budget-table-row']
});