/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2016 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('deployment-cloud', function () {

  juju.components.DeploymentCloud = React.createClass({
    propTypes: {
      acl: React.PropTypes.object.isRequired,
      cloud: React.PropTypes.object,
      getCloudProviderDetails: React.PropTypes.func.isRequired,
      listClouds: React.PropTypes.func.isRequired,
      setCloud: React.PropTypes.func.isRequired
    },

    getInitialState: function () {
      return {
        clouds: [],
        cloudsLoading: true
      };
    },

    componentWillMount: function () {
      this.props.listClouds((error, clouds) => {
        if (error) {
          console.error('Unable to list clouds', error);
          return;
        }
        let cloudList = [];
        if (clouds) {
          cloudList = Object.keys(clouds).map(name => {
            const cloud = clouds[name];
            cloud.name = name;
            return cloud;
          });
        }
        this.setState({
          clouds: cloudList,
          cloudsLoading: false
        });
        // If there is only one cloud then automatically select it.
        if (cloudList.length === 1) {
          this.props.setCloud(cloudList[0]);
        }
      });
    },

    /**
      Generate the list of clouds.
       @method _generateClouds
      @returns {Object} The cloud list.
    */
    _generateClouds: function () {
      if (this.state.cloudsLoading) {
        return React.createElement(
          'div',
          { className: 'deployment-cloud__loading' },
          React.createElement(juju.components.Spinner, null)
        );
      }
      if (this.props.cloud) {
        return;
      }
      var clouds = [];
      this.state.clouds.forEach((cloud, i) => {
        var classes = classNames('deployment-cloud__cloud', 'four-col', { 'last-col': i % 3 === 2 });
        clouds.push(React.createElement(
          'li',
          { className: classes,
            key: cloud.name,
            onClick: this.props.setCloud.bind(null, cloud),
            role: 'button',
            tabIndex: '0' },
          React.createElement(
            'span',
            { className: 'deployment-cloud__cloud-logo' },
            this._generateLogo(cloud)
          )
        ));
      });
      return React.createElement(
        'ul',
        { className: 'deployment-cloud__list' },
        clouds
      );
    },

    /**
      Generate the logo for the selected cloud.
       @method _generateCloud
      @returns {Object} The cloud.
    */
    _generateCloud: function () {
      var cloud = this.props.cloud;
      if (!cloud) {
        return;
      }
      return React.createElement(
        'div',
        { className: 'deployment-cloud__chosen' },
        this._generateLogo(cloud)
      );
    },

    /**
      Generate the logo for a cloud;
       @method _generateLogo
      @param {Object} cloud A cloud.
      @returns {Array} The logo.
    */
    _generateLogo: function (cloud) {
      const info = this.props.getCloudProviderDetails(cloud.cloudType);
      if (!info) {
        return cloud.name;
      }
      return info.showLogo ? React.createElement(juju.components.SvgIcon, {
        height: info.svgHeight,
        name: info.id,
        width: info.svgWidth }) : info.title;
    },

    render: function () {
      return React.createElement(
        'div',
        null,
        this._generateClouds(),
        this._generateCloud()
      );
    }

  });
}, '0.1.0', {
  requires: ['loading-spinner', 'svg-icon']
});