/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2016 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('deployment-services', function () {

  juju.components.DeploymentServices = React.createClass({
    propTypes: {
      acl: React.PropTypes.object.isRequired,
      changesFilterByParent: React.PropTypes.func.isRequired,
      generateAllChangeDescriptions: React.PropTypes.func.isRequired,
      groupedChanges: React.PropTypes.object.isRequired,
      listPlansForCharm: React.PropTypes.func.isRequired,
      servicesGetById: React.PropTypes.func.isRequired,
      showChangelogs: React.PropTypes.bool,
      withPlans: React.PropTypes.bool
    },

    /**
      Create a list of services from the change set.
       @method _getServices
      @returns {Array} A list of services.
    */
    _getServices: function () {
      var addedServices = this.props.groupedChanges['_deploy'];
      if (!addedServices) {
        return [];
      }
      return Object.keys(addedServices).map(change => {
        return this.props.servicesGetById(addedServices[change].command.options.modelId);
      });
    },

    /**
      Generate the list of extra info markup.
       @method _generateExtraInfo
      @returns {Array} A list of elements by service.
    */
    _generateExtraInfo: function () {
      var addedServices = this.props.groupedChanges['_deploy'] || [];
      var infos = {};
      Object.keys(addedServices).forEach(change => {
        var serviceId = addedServices[change].command.options.modelId;
        var changeId = addedServices[change].id;
        var changes = this.props.changesFilterByParent(changeId);
        var descriptions = this.props.generateAllChangeDescriptions(changes);
        var items = descriptions.map(change => {
          return React.createElement(juju.components.DeploymentChangeItem, {
            change: change,
            key: change.id,
            showTime: false });
        });
        infos[serviceId] = React.createElement(
          'ul',
          { className: 'deployment-services__changes' },
          items
        );
      });
      return infos;
    },

    /**
      Generate spend summary.
       @method _generateSpend
      @returns {Object} The spend markup.
    */
    _generateSpend: function () {
      if (!this.props.withPlans) {
        return;
      }
      return React.createElement(
        'div',
        { className: 'prepend-seven' },
        'Maximum monthly spend:\xA0',
        React.createElement(
          'span',
          { className: 'deployment-services__max' },
          '$100'
        )
      );
    },

    render: function () {
      return React.createElement(
        'div',
        null,
        React.createElement(juju.components.BudgetTable, {
          acl: this.props.acl,
          allocationEditable: true,
          extraInfo: this._generateExtraInfo(),
          listPlansForCharm: this.props.listPlansForCharm,
          plansEditable: true,
          services: this._getServices(),
          showExtra: this.props.showChangelogs,
          withPlans: this.props.withPlans }),
        this._generateSpend()
      );
    }

  });
}, '0.1.0', {
  requires: ['budget-table', 'deployment-change-item']
});