/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2016 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('deployment-signup', function () {

  juju.components.DeploymentSignup = React.createClass({
    propTypes: {
      changeState: React.PropTypes.func.isRequired,
      exportEnvironmentFile: React.PropTypes.func.isRequired,
      modelName: React.PropTypes.string.isRequired,
      staticURL: React.PropTypes.string
    },

    /**
      Set the cookie for hiding the sign-up page.
       @method _setBetaCookie
      @param {Boolean} permanent Whether the page must be permanently hidden.
    */
    _setBetaCookie: function (permanent) {
      const expiration = new Date();
      if (permanent) {
        expiration.setFullYear(2020);
      } else {
        expiration.setHours(expiration.getHours() + 3);
      }
      document.cookie = 'beta-signup-seen=true; expires=' + expiration.toUTCString();
    },

    /**
      Handle navigating to the deployment flow.
       @method _displayFlow
    */
    _displayFlow: function () {
      // TODO: In the future the deployment flow must be opened in sandbox mode
      // at demo.jujucharms.com.
      this.props.changeState({
        sectionC: {
          component: 'deploy',
          metadata: {
            activeComponent: 'flow'
          }
        }
      });
      this._setBetaCookie(false);
    },

    /**
      Handle the email registration.
       @method _handleSignup
    */
    _handleSignup: function () {
      this._setBetaCookie(true);
      this.props.changeState({
        sectionC: {
          component: 'deploy',
          metadata: {
            activeComponent: 'flow'
          }
        }
      });
    },

    render: function () {
      return React.createElement(
        juju.components.DeploymentPanel,
        {
          changeState: this.props.changeState,
          title: this.props.modelName },
        React.createElement(
          'div',
          { className: 'deployment-signup' },
          React.createElement(
            'div',
            { className: 'row border-bottom' },
            React.createElement(
              'h2',
              null,
              'Install Juju to deploy locally'
            ),
            React.createElement(
              'div',
              { className: 'six-col' },
              React.createElement(
                'p',
                { className: 'intro' },
                'Local deployment uses LXD containers, allowing you to recreate your production environment on your own machine. This minimises portability issues when deploying to a public cloud, OpenStack or bare metal.'
              ),
              React.createElement(
                'p',
                { className: 'intro' },
                'To deploy locally:'
              ),
              React.createElement(
                'ol',
                { className: 'deployment-signup__numbered-list' },
                React.createElement(
                  'li',
                  { className: 'deployment-signup__numbered-list-item' },
                  'Download your model'
                ),
                React.createElement(
                  'li',
                  { className: 'deployment-signup__numbered-list-item' },
                  React.createElement(
                    'a',
                    { href: 'https://jujucharms.com/docs' },
                    'Install Juju\xA0\u203A'
                  )
                ),
                React.createElement(
                  'li',
                  { className: 'deployment-signup__numbered-list-item' },
                  'Add your model to deploy'
                )
              ),
              React.createElement(
                'p',
                null,
                'Continue to the\xA0',
                React.createElement(juju.components.GenericButton, {
                  action: this._displayFlow,
                  type: 'inline-neutral',
                  title: 'Deployment demo of Juju' })
              )
            ),
            React.createElement(
              'div',
              { className: 'prepend-one four-col last-col' },
              React.createElement(juju.components.SvgIcon, {
                className: 'juju-logo',
                name: 'juju-logo',
                size: '100%' })
            )
          ),
          React.createElement(
            'div',
            { className: 'row' },
            React.createElement(
              'h2',
              null,
              'A new way to deploy'
            ),
            React.createElement(
              'div',
              { className: 'six-col' },
              React.createElement(
                'p',
                null,
                'Coming soon: deploy from hosted Juju direct to public clouds. For early access to this feature, sign up for the beta.'
              ),
              React.createElement(
                'ul',
                null,
                React.createElement(
                  'li',
                  null,
                  'Deploy to all major public clouds directly from your browser'
                ),
                React.createElement(
                  'li',
                  null,
                  'Hosted and managed Juju controllers'
                ),
                React.createElement(
                  'li',
                  null,
                  'Identity management across all models'
                ),
                React.createElement(
                  'li',
                  null,
                  'Reusable shareable models with unlimited users'
                )
              ),
              React.createElement(
                'p',
                null,
                React.createElement(
                  'a',
                  { href: 'https://jujucharms.com/beta',
                    target: '_blank',
                    className: 'button--inline-positive',
                    onClick: this._handleSignup },
                  'Sign up for early access'
                )
              )
            ),
            React.createElement(
              'div',
              { className: 'six-col last-col' },
              React.createElement(
                'ul',
                { className: 'inline-logos no-bullets' },
                React.createElement(
                  'li',
                  { className: 'inline-logos__item' },
                  React.createElement(juju.components.SvgIcon, {
                    className: 'inline-logos__image',
                    name: 'aws',
                    size: '100%' })
                ),
                React.createElement(
                  'li',
                  { className: 'inline-logos__item' },
                  React.createElement(juju.components.SvgIcon, {
                    className: 'inline-logos__image',
                    name: 'google',
                    size: '100%' })
                ),
                React.createElement(
                  'li',
                  { className: 'inline-logos__item' },
                  React.createElement(juju.components.SvgIcon, {
                    className: 'inline-logos__image',
                    name: 'azure',
                    size: '100%' })
                )
              )
            )
          )
        )
      );
    }

  });
}, '0.1.0', {
  requires: ['deployment-panel', 'generic-button', 'generic-input']
});