/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2016 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

chai.config.includeStack = true;
chai.config.truncateThreshold = 0;

describe('DeploymentSignup', function () {

  beforeAll(function (done) {
    // By loading this file it adds the component to the juju components.
    YUI().use('deployment-signup', function () {
      done();
    });
  });

  it('can render', function () {
    const changeState = sinon.stub();
    const exportEnvironmentFile = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(
      juju.components.DeploymentSignup,
      {
        changeState: changeState,
        exportEnvironmentFile: exportEnvironmentFile,
        modelName: 'Prawns on the barbie' },
      React.createElement(
        'span',
        null,
        'content'
      )
    ), true);
    var instance = renderer.getMountedInstance();
    var output = renderer.getRenderOutput();
    const expected = React.createElement(
      juju.components.DeploymentPanel,
      {
        changeState: changeState,
        title: 'Prawns on the barbie' },
      React.createElement(
        'div',
        { className: 'deployment-signup' },
        React.createElement(
          'div',
          { className: 'row border-bottom' },
          React.createElement(
            'h2',
            null,
            'Install Juju to deploy locally'
          ),
          React.createElement(
            'div',
            { className: 'six-col' },
            React.createElement(
              'p',
              { className: 'intro' },
              'Local deployment uses LXD containers, allowing you to recreate your production environment on your own machine. This minimises portability issues when deploying to a public cloud, OpenStack or bare metal.'
            ),
            React.createElement(
              'p',
              { className: 'intro' },
              'To deploy locally:'
            ),
            React.createElement(
              'ol',
              { className: 'deployment-signup__numbered-list' },
              React.createElement(
                'li',
                { className: 'deployment-signup__numbered-list-item' },
                'Download your model'
              ),
              React.createElement(
                'li',
                { className: 'deployment-signup__numbered-list-item' },
                React.createElement(
                  'a',
                  { href: 'https://jujucharms.com/docs' },
                  'Install Juju\xA0\u203A'
                )
              ),
              React.createElement(
                'li',
                { className: 'deployment-signup__numbered-list-item' },
                'Add your model to deploy'
              )
            ),
            React.createElement(
              'p',
              null,
              'Continue to the\xA0',
              React.createElement(juju.components.GenericButton, {
                action: instance._displayFlow,
                type: 'inline-neutral',
                title: 'Deployment demo of Juju' })
            )
          ),
          React.createElement(
            'div',
            { className: 'prepend-one four-col last-col' },
            React.createElement(juju.components.SvgIcon, {
              className: 'juju-logo',
              name: 'juju-logo',
              size: '100%' })
          )
        ),
        React.createElement(
          'div',
          { className: 'row' },
          React.createElement(
            'h2',
            null,
            'A new way to deploy'
          ),
          React.createElement(
            'div',
            { className: 'six-col' },
            React.createElement(
              'p',
              null,
              'Coming soon: deploy from hosted Juju direct to public clouds. For early access to this feature, sign up for the beta.'
            ),
            React.createElement(
              'ul',
              null,
              React.createElement(
                'li',
                null,
                'Deploy to all major public clouds directly from your browser'
              ),
              React.createElement(
                'li',
                null,
                'Hosted and managed Juju controllers'
              ),
              React.createElement(
                'li',
                null,
                'Identity management across all models'
              ),
              React.createElement(
                'li',
                null,
                'Reusable shareable models with unlimited users'
              )
            ),
            React.createElement(
              'p',
              null,
              React.createElement(
                'a',
                { href: 'https://jujucharms.com/beta',
                  target: '_blank',
                  className: 'button--inline-positive',
                  onClick: instance._handleSignup },
                'Sign up for early access'
              )
            )
          ),
          React.createElement(
            'div',
            { className: 'six-col last-col' },
            React.createElement(
              'ul',
              { className: 'inline-logos no-bullets' },
              React.createElement(
                'li',
                { className: 'inline-logos__item' },
                React.createElement(juju.components.SvgIcon, {
                  className: 'inline-logos__image',
                  name: 'aws',
                  size: '100%' })
              ),
              React.createElement(
                'li',
                { className: 'inline-logos__item' },
                React.createElement(juju.components.SvgIcon, {
                  className: 'inline-logos__image',
                  name: 'google',
                  size: '100%' })
              ),
              React.createElement(
                'li',
                { className: 'inline-logos__item' },
                React.createElement(juju.components.SvgIcon, {
                  className: 'inline-logos__image',
                  name: 'azure',
                  size: '100%' })
              )
            )
          )
        )
      )
    );
    assert.deepEqual(output, expected);
  });

  it('can navigate to the flow view', function () {
    const changeState = sinon.stub();
    const output = jsTestUtils.shallowRender(React.createElement(
      juju.components.DeploymentSignup,
      {
        changeState: changeState,
        exportEnvironmentFile: sinon.stub(),
        modelName: 'Lamington' },
      React.createElement(
        'span',
        null,
        'content'
      )
    ));
    output.props.children.props.children[0].props.children[1].props.children[3].props.children[1].props.action();
    assert.equal(changeState.callCount, 1);
    assert.deepEqual(changeState.args[0][0], {
      sectionC: {
        component: 'deploy',
        metadata: {
          activeComponent: 'flow'
        }
      }
    });
  });
});