/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2016 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

describe('ISVProfile', () => {

  beforeAll(done => {
    // By loading this file it adds the component to the juju components.
    YUI().use('isv-profile', () => {
      done();
    });
  });

  it('renders the navigation', () => {
    var component = jsTestUtils.shallowRender(React.createElement(juju.components.ISVProfile, null), true);
    var output = component.getRenderOutput();
    var content = output.props.children.props.children.props.children[0];
    var expected = React.createElement(
      'nav',
      { className: 'three-col isv-profile__navigation' },
      React.createElement(
        'ul',
        { className: 'isv-profile__navigation-list' },
        React.createElement(
          'li',
          { className: 'isv-profile__navigation-item--title' },
          React.createElement(
            'a',
            { href: '' },
            'Acme Corp'
          )
        ),
        React.createElement(
          'li',
          { className: 'isv-profile__navigation-item' },
          React.createElement(
            'a',
            { href: '' },
            'Charms'
          )
        ),
        React.createElement(
          'li',
          { className: 'isv-profile__navigation-item' },
          React.createElement(
            'a',
            { href: '' },
            'Issues'
          )
        ),
        React.createElement(
          'li',
          { className: 'isv-profile__navigation-item' },
          React.createElement(
            'a',
            { href: '' },
            'Revenue'
          )
        )
      )
    );
    assert.deepEqual(content, expected);
  });

  it('renders the live data section', () => {
    var component = jsTestUtils.shallowRender(React.createElement(juju.components.ISVProfile, null), true);
    var output = component.getRenderOutput();
    var wrapper = output.props.children.props.children;
    var content = wrapper.props.children[1].props.children[0];
    var expected = React.createElement(
      'div',
      { className: 'isv-profile__live-data clearfix' },
      React.createElement(
        'h3',
        { className: 'isv-profile__section-title' },
        '03 September 2016 - 14:35'
      ),
      React.createElement(
        'div',
        { className: 'three-col isv-profile__box' },
        React.createElement(
          'h3',
          { className: 'isv-profile__box-title' },
          'User this hour'
        ),
        React.createElement(
          'p',
          { className: 'isv-profile__box-stat' },
          '23'
        )
      ),
      React.createElement(
        'div',
        { className: 'three-col isv-profile__box' },
        React.createElement(
          'h3',
          { className: 'isv-profile__box-title' },
          'Units this hour'
        ),
        React.createElement(
          'p',
          { className: 'isv-profile__box-stat' },
          '64'
        )
      ),
      React.createElement(
        'div',
        { className: 'three-col last-col isv-profile__box' },
        React.createElement(
          'h3',
          { className: 'isv-profile__box-title' },
          'Net revenue to date'
        ),
        React.createElement(
          'p',
          { className: 'isv-profile__box-stat' },
          '$1,500.00'
        )
      )
    );
    assert.deepEqual(content, expected);
  });
});